/*
 * Copyright (c) 1994 Vectaport Inc., Cartoactive Systems
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Dlg/dlgcov.h>
#include <Dlg/dlgdb.h>
#include <Dlg/dlglib.h>

#include <OS/directory.h>
#include <OS/string.h>

#include <stdio.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/stat.h>
#include <string.h>

/*****************************************************************************/

DlgLibrary::DlgLibrary (DlgDatabase* database, const char* name) 
: MapLibrary(database,name) {
}

DlgLibrary::~DlgLibrary ()
{
}

DlgCoverage* DlgLibrary::coverage(unsigned covnum) const {
    return (DlgCoverage*)MapLibrary::coverage(covnum);
}

DlgCoverage* DlgLibrary::coverage(const char* name) const {
    return (DlgCoverage*)MapLibrary::coverage(name);
}

char** DlgLibrary::coverage_names (int &ncovs) const {
    int nc;
    char **covnms;
    char *covnames100k[] = { "hydro", "mtran", "rail", "roads" };
    char *covnames2m[] =
    { "ab", "cf", "pb", "rd", "rr",
      "st_a", "st_b", "st_c", "st_d", "st_e", "st_f", "st_g", "st_h", "st",
      "wb_a", "wb_b", "wb_c", "wb_d", "wb_e", "wb_f", "wb",
      "hp" };
    if (((DlgDatabase*)database())->Is100K()) {
	nc = 4;
	covnms = covnames100k;
    }
    else {
	nc = 22;
	covnms = covnames2m;
    }
    char **coverages = new char*[nc];
    DIR* dir;
    struct dirent* entry;
    ncovs = 0;
    dir = opendir(path());
    if (dir == NULL) return nil;
    while ((entry = readdir(dir)) != NULL) {
	if ((strcmp(entry->d_name,".") != 0) &&
	    (strcmp(entry->d_name,"..") != 0)) {
	    for (int i = 0; i < nc; i++)
		if (strstr(entry->d_name, covnms[i])) {
		    coverages[ncovs++] = strdup(covnms[i]);
		    break;
		}
	}
    }
    closedir(dir);
    return coverages;
}

MapCoverage* DlgLibrary::make_coverage(const char* name) { 
    return new DlgCoverage(this,name);
}
					   

