/*
 * Copyright 1997 Vectaport Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software and
 * its documentation for any purpose is hereby granted without fee, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of the copyright holders not be used in
 * advertising or publicity pertaining to distribution of the software
 * without specific, written prior permission.  The copyright holders make
 * no representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
 * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS.
 * IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY SPECIAL,
 * INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING
 * FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT,
 * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
 * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include <Map/mapelt.h>
#include <Map/mapprim.h>
#include <MapUnidraw/mapcomps.h>
#include <MapUnidraw/mapeditor.h>
#include <MapUnidraw/maptools.h>
#include <MapUnidraw/mapviewstate.h>
#include <OverlayUnidraw/ovviewer.h>
#include <Unidraw/clipboard.h>
#include <Unidraw/ctrlinfo.h>
#include <Unidraw/manips.h>
#include <InterViews/event.h>
#include <InterViews/transformer.h>
#include <IV-2_6/InterViews/rubcurve.h>
#include <IV-2_6/InterViews/rubverts.h>

/****************************************************************************/

LineFeatureTool::LineFeatureTool(ControlInfo* ci) : Tool(ci) {}

Manipulator* LineFeatureTool::CreateManipulator(
    Viewer* v, Event& e, Transformer* rel
) {
    Manipulator* m = nil;

    v->Constrain(e.x, e.y);
    Coord x[1], y[1];
    x[0] = e.x;
    y[0] = e.y;
    GrowingVertices* rub = new GrowingMultiLine(
            nil, nil, x, y, 1, -1, HANDLE_SIZE
	    );
    m = new VertexManip(
	        v, rub, rel, this, DragConstraint(HorizOrVert | Gravity)
		);

    return m;
}

Command* LineFeatureTool::InterpretManipulator(Manipulator* m) {
    DragManip* dm = (DragManip*) m;
    MapEditor* ed = (MapEditor*)dm->GetViewer()->GetEditor();
    Transformer* rel = dm->GetTransformer();
    Command* cmd = nil;
    Transformer* trans = ed->GetMapViewState()->deftrans();

    GrowingVertices* gv = (GrowingVertices*) dm->GetRubberband();
    Coord* x, *y;
    int n, pt;
    gv->GetCurrent(x, y, n, pt);

    if (n > 2 || x[0] != x[1] || y[0] != y[1]) {
      if (rel != nil) {
	rel = new Transformer(rel);
	rel->Invert();
      }
      float* dx = new float[n];
      float* dy = new float[n];
      float* utmx = new float[n];
      float* utmy = new float[n];
      for (int i = 0; i < n; i++)
	rel->transform(x[i], y[i], dx[i], dy[i]);
      if (trans) {
	for (int j = 0; j < n; j++)
	  trans->inverse_transform(dx[j], dy[j], utmx[j], utmy[j]);
	delete [] dx;
	delete [] dy;
      }
      else {
	delete [] utmx;
	delete [] utmy;
	utmx = dx;
	utmy = dy;
      }
      //double* ex = new double[n];
      //ouble* ey = new double[n];
      //for (int i = 0; i < n; i++) {
      //ex[i] = utmx[i];
      //ey[i] = utmy[i];
      //}
      MapFeature* line = new MapFeature(0, MapFeature::LineType);
      MapEdge* edge = new MapEdge(line);
      edge->insert_points(n, utmx, utmy);
      delete [] utmx;
      delete [] utmy;
      line->edge_primitive(edge);
      cmd = new PasteCmd(ed, new Clipboard(new MapFeatureComp(line)));
    }
    delete x;
    delete y;
    return cmd;
}
